DropdownCustomStyle

Struct DropdownCustomStyle 

pub struct DropdownCustomStyle(/* private fields */);
A struct that allows for easy custom styling of the Dropdown using the Dropdown::dropdown_style method or the Style::custom_style method.

Implementations

Source

impl DropdownCustomStyle

pub fn new() -> DropdownCustomStyle

Creates a new DropDownCustomStyle with default values.

pub fn close_on_accept(self, close: bool) -> DropdownCustomStyle

Sets the CloseOnAccept property for the dropdown, which determines whether the dropdown should automatically close when an item is selected. The default value is true.

Arguments
  • close: If set to true, the dropdown will close upon item selection. If false, it will remain open after an item is selected.